from pathlib import Path
from src.config.config import Configuration
from src.rendering.renderer import render_page, render_error_page
from flask import send_file, request
from src.rendering.image import generate_thumbnail
import os


class RouteManager:
    def __init__(self, config: Configuration):
        self.config = config

    def _validate_and_sanitize_path(self, base_dir, requested_path):
        """
        Validate and sanitize the requested path to ensure it does not traverse above the base directory.

        :param base_dir: The base directory that the requested path should be within.
        :param requested_path: The requested file path to validate.
        :return: A secure version of the requested path if valid, otherwise None.
        """
        # Normalize both paths
        base_dir = Path(base_dir)
        requested_path: Path = base_dir / requested_path

        # Check if the requested path is within the base directory
        if requested_path < base_dir:
            return None

        # Ensure the path does not contain any '..' or '.' components
        secure_path = os.path.relpath(requested_path, base_dir)
        secure_path_parts = secure_path.split(os.sep)

        for part in secure_path_parts:
            if part == "." or part == "..":
                print("Illegal path nice try")
                return None

        # Reconstruct the secure path
        secure_path = os.path.join(base_dir, *secure_path_parts)
        secure_path = Path(secure_path)

        # Check if path exists
        if not secure_path.exists():
            raise Exception("Illegal path")

        for part in secure_path.parts:
            if part.startswith("___"):
                print("hidden file")
                raise Exception("Illegal path")

        return secure_path

    def _ensure_route(self, path: str):
        file_path: Path = self.config.content_dir / (path if path else "index.md")
        if file_path < self.config.content_dir:
            raise Exception("Illegal path")

        if not self._validate_and_sanitize_path(
            self.config.content_dir, str(file_path)
        ):
            raise Exception("Illegal path")

    def default_route(self, path: str):
        try:
            self._ensure_route(path)
        except Exception as e:
            return render_error_page(
                404,
                "Not Found",
                "The requested resource was not found on this server.",
                self.config.templates_dir,
            )
        file_path: Path = self.config.content_dir / (path if path else "index.md")
        return render_page(
            file_path,
            base_path=self.config.content_dir,
            template_path=self.config.templates_dir,
            style_path=self.config.styles_dir,
        )

    def get_style(self, path: str):
        try:
            self._validate_and_sanitize_path(self.config.styles_dir, path)
        except Exception as e:
            return render_error_page(
                404,
                "Not Found",
                f"The requested resource was not found on this server. {e}",
                self.config.templates_dir,
            )
        file_path: Path = self.config.styles_dir / path
        if file_path.exists():
            return send_file(file_path)
        else:
            return render_error_page(
                404,
                "Not Found",
                "The requested resource was not found on this server.",
                self.config.templates_dir,
            )

    def get_static(self, path: str):
        try:
            self._validate_and_sanitize_path(self.config.content_dir, path)
        except Exception as e:
            return render_error_page(
                404,
                "Not Found",
                "The requested resource was not found on this server.",
                self.config.templates_dir,
            )
        file_path: Path = self.config.content_dir / path
        if file_path.exists():
            # Check to see if the file is an image, if it is, render a thumbnail
            if file_path.suffix.lower() in [".jpg", ".jpeg", ".png", ".gif"]:
                max_width = request.args.get("max_width", default=2048, type=int)
                thumbnail_bytes, img_format = generate_thumbnail(
                    str(file_path), 10, 2048, max_width
                )
                return (
                    thumbnail_bytes,
                    200,
                    {"Content-Type": f"image/{img_format.lower()}",
                     "cache-control": "public, max-age=31536000"},
                )
            return send_file(file_path)
        else:
            return render_error_page(
                404,
                "Not Found",
                "The requested resource was not found on this server.",
                self.config.templates_dir,
            )
